package org.jastadd.ros2rag.tests;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

/**
 * Utility methods for tests.
 *
 * @author rschoene - Initial contribution
 */
public class TestUtils {

  public static final double DELTA = 0.001d;

  public static String getMqttHost() {
    if (System.getenv("GITLAB_CI") != null) {
      // we are in the CI, so use "mqtt" as host
      return "mqtt";
    } {
      // else assume a locally running mqtt broker
      return "localhost";
    }
  }

  public static int getMqttDefaultPort() {
    return 1883;
  }

  public static int exec(Class<?> klass, String[] args, File err) throws IOException,
      InterruptedException {
    String javaHome = System.getProperty("java.home");
    String javaBin = javaHome + File.separator + "bin" + File.separator + "java";
    String classpath = System.getProperty("java.class.path");
    String className = klass.getName();

    String[] newArgs = new String[args.length + 4];
    newArgs[0] = javaBin;
    newArgs[1] = "-cp";
    newArgs[2] = classpath;
    newArgs[3] = className;
    System.arraycopy(args, 0, newArgs, 4, args.length);

    ProcessBuilder builder = new ProcessBuilder(newArgs);
//    builder.redirectOutput(err);
    builder.redirectError(err);

    Process process = builder.start();
    process.waitFor();
    return process.exitValue();
  }

  public static String readFile(String path, Charset encoding)
      throws IOException {
    byte[] encoded = Files.readAllBytes(Paths.get(path));
    return new String(encoded, encoding);
  }

  static void waitForMqtt() throws InterruptedException {
    TimeUnit.SECONDS.sleep(2);
  }
}
